function[U,coretensor]=m_mode_svd(dtensor,modes,big_flag); 
% This implements the N-Mode SVD as described in the ECCV'02 paper.
% "dtensor" is a data tensor or n-way array
% "modes" is an array which specifies which mode matrices ought to be computed
% "big_flag" the default is 1. "Big_Flag" should be set to 1 if you are running 
% out of memory, otherwise it should be set to 0 which optimize for speed.  
%
% Example:
%	 randn('state',0)
%	 dtensor=randn(3,4,5);
%	 [U,core]=tensor_decomposition(dtensor);
%	 U
%	U = 
%
%	    [3x3 double]    [4x4 double]    [5x5 double]
%
%	 size(core)
%       ans =
%             [3 4 5]
%
% Example: Note the size of mode 3. U{3} is 15 by 12 and not 15 by 15. 
%          Note the core size as well.
%	 randn('state',0)
%	 dtensor=randn(3,4,15);
%	 [U,core]=tensor_decomposition(dtensor);
%	 U
%	U = 
%
%	    [3x3 double]    [4x4 double]    [15x12 double]
%
%	 size(core)
%       ans =
%             [3 4 12]
%
% Example: Compute a subset of the mode matrices
%
%	 randn('state',0)
%	 dtensor=randn(3,4,5);
%	 [U,core]=tensor_decomposition(dtensor,[1 3]);
%        U
%	U = 
%	    [3x3 double]    [1]    [5x5 double]
%
%	 U{2}
%       ans =
%             1
%
% Only U{1} and U{3} are computed. U{2} is equal with 1, which is a
% holder value. 
%
%
% Example: Decompose the tensor when the observation is a vector rather than 
%          a scalar. In this case the observation mode is not decomposed out
%          of the tensor.  If the observation mode is mode 3 then you should 
%          compute the first two modes and an extendedcore
%
%	 randn('state',0)
%	 dtensor=randn(3,4,15);
%	 [U,extendedcore]=tensor_decomposition(dtensor,[1 2]);
%        U
%	U = 
%	    [3x3 double]    [4x4]
%
%	 size(extendedcore)
%       ans=
%            [3 4 15]
%
% Only U{1} and U{3} are computed. U{2} is equal with 1, which is a
% holder value. 
%
% Please report bugs/fixes to maov@mrl.nyu.edu
% Copyright 2005-2025 M.A.O.Vasilescu
% Please do NOT redistribute without permission; direct requests to maov@mrl.nyu.edu

% Paper ECCV'02, vol.2350, pg.447-460./Paper ICPR'02, vol.3 pg.456-460.
% Paper CVPR'03, vol.2, pg.93-99.
% Created 2001 and Revised Feb 28, 2005 by M.A.O.Vasilescu



[d]=size(dtensor);
%keyboard 
if nargin==1
  modes=1:length(d);
  big_flag=1;
elseif nargin==2
  for i=1:length(d), U{i}=1; end
  big_flag=1;   
end

cdata=dtensor;
%clear dtensor
pack
for i=modes
  ['Computing mode ' int2str(i) '.']
  %M=double(matricize(dtensor,i));
  %[r c]=size(M);

  M=double(matricize(cdata,i));
  [r c]=size(M);
  
  %M=M-repmat(mean(M,2),1,c);

  pack
  [u]=tfastsvd(M,big_flag);
  U{i}=u;

  clear u M
  %'core'
  if prod(d([1:(i-1) (i+1):length(d)]))>350000
    pack
    cdata=mode_m_prod(double(cdata),U{i}',i,1);
  else
    pack
    cdata=mode_m_prod(double(cdata),U{i}',i,0);
  end
end
coretensor=cdata;
clear cdata i d modes 





